@@ -27,6 +27,7 @@
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <unistd.h>
 
 #define BUFSZ 0x8000
 
@@ -69,19 +70,32 @@ int http_send(struct tunnel *tunnel, const char *request, ...)
 {
 	va_list args;
 	char buffer[BUFSZ];
+	char logbuffer[BUFSZ];
 	int length;
 	int n = 0;
 
 	va_start(args, request);
 	length = vsnprintf(buffer, BUFSZ, request, args);
 	va_end(args);
+	strcpy(logbuffer,buffer);
+	if (loglevel <= OFV_LOG_DEBUG_DETAILS && tunnel->config->password[0]!='\0') {
+		char* pwstart;
+		pwstart = strstr(logbuffer, tunnel->config->password);
+		if (pwstart != NULL) {
+			int pos, pwlen, i;
+			pos = pwstart - logbuffer;
+			pwlen = strlen(tunnel->config->password);
+			for (i=pos; i<pos+pwlen; i++)
+				logbuffer[i]='*';
+		}
+	}
 
 	if (length < 0)
 		return ERR_HTTP_INVALID;
 	else if (length >= BUFSZ)
 		return ERR_HTTP_TOO_LONG;
 
-	log_debug_details("%s: \n%s\n", __func__, buffer);
+	log_debug_details("%s: \n%s\n", __func__, logbuffer);
 
 	while (n == 0)
 		n = safe_ssl_write(tunnel->ssl_handle, (uint8_t *) buffer,
@@ -373,6 +387,14 @@ static int get_auth_cookie(
 	return ret;
 }
 
+static void delay_otp(struct tunnel *tunnel)
+{
+	if (tunnel->config->otp_delay > 0) {
+		log_info("Delaying OTP by %d seconds...\n", tunnel->config->otp_delay);
+		sleep(tunnel->config->otp_delay);
+	}
+}
+
 static
 int try_otp_auth(
         struct tunnel *tunnel,
@@ -410,6 +432,8 @@ int try_otp_auth(
 	 * Fall back to default prompt if not found/parseable
 	 */
 	p = strstr(s, "Please");
+	if (tunnel->config->otp_prompt != NULL)
+		p = strstr(s, tunnel->config->otp_prompt);
 	if (p) {
 		e = strchr(p, '<');
 		if (e != NULL) {
@@ -489,7 +513,8 @@ int try_otp_auth(
 			size_t l;
 			v = NULL;
 			if (cfg->otp[0] == '\0') {
-				read_password(p, cfg->otp, FIELD_SIZE);
+				read_password(cfg->pinentry, "otp",
+				              p, cfg->otp, FIELD_SIZE);
 				if (cfg->otp[0] == '\0') {
 					log_error("No OTP specified\n");
 					return 0;
@@ -558,6 +583,8 @@ int auth_log_in(struct tunnel *tunnel)
 
 	/* Probably one-time password required */
 	if (strncmp(res, "HTTP/1.1 401 Authorization Required\r\n", 37) == 0) {
+		delay_otp(tunnel);
+
 		ret = try_otp_auth(tunnel, res, &res, &response_size);
 		if (ret != 1)
 			goto end;
@@ -598,7 +625,8 @@ int auth_log_in(struct tunnel *tunnel)
 		get_value_from_response(res, "polid=", polid, 32);
 
 		if (cfg->otp[0] == '\0') {
-			read_password("Two-factor authentication token: ",
+			read_password(cfg->pinentry, "otp",
+			              "Two-factor authentication token: ",
 			              cfg->otp, FIELD_SIZE);
 			if (cfg->otp[0] == '\0') {
 				log_error("No token specified\n");
@@ -612,6 +640,7 @@ int auth_log_in(struct tunnel *tunnel)
 		         "&redir=%%2Fremote%%2Findex&just_logged_in=1",
 		         username, realm, reqid, polid, group, tokenresponse);
 
+		delay_otp(tunnel);
 		ret = http_request(
 		              tunnel, "POST", "/remote/logincheck",
 		              data, &res, &response_size);
